@@ -7,7 +7,7 @@ use strict;
 use threads (
     'yield',
     'stringify',
-    'stack_size' => 16 * 4096, # No need for a large stack
+    'stack_size' => 4 * 4096, # No need for a large stack
     'exit'       => 'threads_only',
 );
 use threads::shared;
@@ -17,9 +17,10 @@ use List::Util qw(shuffle);
 use Getopt::Long;
 use Pod::Usage;
 use Sys::CPU;
+use File::Basename;
 
 ## I only use this for debugging
-# use Data::Dumper::Simple; $Data::Dumper::Sortkeys = 1; $Data::Dumper::Purity = 1;
+# use Data::Dumper::Simple; $Data::Dumper::Sortkeys = 1; $Data::Dumper::Purity = 1; $Data::Dumper::Deepcopy = 1;
 
 my $errors           = FALSE;
 my $auto             = FALSE;
@@ -27,6 +28,7 @@ my $showall          = FALSE;
 my $help             = FALSE;
 my $delay            = 3;
 my $nosplash         = FALSE;
+my $show_names       = FALSE;
 my $noaccel          = FALSE;
 my $threads          = Sys::CPU::cpu_count() * 2;
 my $RUNNING : shared = TRUE;
@@ -46,8 +48,9 @@ GetOptions(
     'nosplash'     => \$nosplash,
     'noaccel'      => \$noaccel,
     'threads=i'    => \$threads,
+    'names'        => \$show_names,
 );
-my @paths = @ARGV;
+my @paths : shared = @ARGV;
 
 unless (scalar(@paths) && !$help) {
     push(@paths,$default_path);
@@ -75,20 +78,21 @@ foreach my $dev (0 .. 31) {
 $SIG{'QUIT'} = \&finish;
 $SIG{'INT'}  = \&finish;
 $SIG{'KILL'} = \&finish;
+
 my $p = gather(@paths);
 
 if ($errors) {
     print STDERR qq{
 
-AUTO            = $auto
-ERRORS          = $errors
-SHOWALL         = $showall
-DELAY           = $delay
-NOSPLASH        = $nosplash
-CPU             = }, Sys::CPU::cpu_type(), qq{
-THREADS         = $threads
-DEVICES         = }, join(', ',@devs), qq{
-PATH(s)         = }, join('; ', @paths), "\n";
+        AUTO            = $auto
+        ERRORS          = $errors
+        SHOWALL         = $showall
+        DELAY           = $delay
+        NOSPLASH        = $nosplash
+        CPU             = }, Sys::CPU::cpu_type(), qq{
+        THREADS         = $threads
+        DEVICES         = }, join(', ',@devs), qq{
+        PATH(s)         = }, join('; ', @paths), "\n";
 
     sleep 5;
 } ## end if ($errors)
@@ -102,7 +106,7 @@ my $showit = $splash;
 my @params;
 for (my $t = 0; $t < $threads; $t++) {
     foreach my $f (@devs) {
-        $params[$t] = (defined($new_x)) ? [$f, $p, $threads, $t, $showit, $delay, $new_x, $new_y] : [$f, $p, $threads, $t, $showit, $delay];
+        $params[$t] = (defined($new_x)) ? [$f, $p, $threads, $t, $showit, $delay, $new_x, $new_y] : [$f, $p, $threads, $t, $showit, $delay, $show_names];
         $thrd[$t] = threads->create({'context' => 'scalar'}, \&show, @{ $params[$t] });
     }
     sleep $showit if ($showit);
@@ -374,6 +378,7 @@ sub show {
     my $job       = shift;
     my $display   = shift;
     my $delay     = shift;
+    my $show_name = shift || 0;
     my ($nx, $ny) = @_;
 
     local $SIG{'ALRM'} = undef;
@@ -383,23 +388,23 @@ sub show {
 
     my $FB = (defined($nx)) ?
       Graphics::Framebuffer->new(
-        'SHOW_ERRORS' => $errors,
-        'RESET'       => 1,
-        'SPLASH'      => $splash,
-        'ACCELERATED' => !$noaccel,
-        'FB_DEVICE'   => $dev,
-        'SPLASH'      => $display,
-        'SIMULATED_X' => $nx,
-        'SIMULATED_Y' => $ny,
+          'SHOW_ERRORS' => $errors,
+          'RESET'       => 1,
+          'SPLASH'      => $splash,
+          'ACCELERATED' => !$noaccel,
+          'FB_DEVICE'   => $dev,
+          'SPLASH'      => $display,
+          'SIMULATED_X' => $nx,
+          'SIMULATED_Y' => $ny,
       )
       :
       Graphics::Framebuffer->new(
-        'SHOW_ERRORS' => $errors,
-        'RESET'       => 1,
-        'SPLASH'      => $splash,
-        'ACCELERATED' => !$noaccel,
-        'FB_DEVICE'   => $dev,
-        'SPLASH'      => $display,
+          'SHOW_ERRORS' => $errors,
+          'RESET'       => 1,
+          'SPLASH'      => $splash,
+          'ACCELERATED' => !$noaccel,
+          'FB_DEVICE'   => $dev,
+          'SPLASH'      => $display,
       );
     $FB->wait_for_console(1);
     $FB->set_color({ 'red' => 0, 'green' => 0, 'blue' => 0, 'alpha' => 255 });
@@ -407,14 +412,26 @@ sub show {
     my $p    = scalar(@pics);
     my $idx  = 0;
     my ($X, $Y, $W, $H) = calculate_window($jobs, $job, $FB->{'XRES'}, $FB->{'YRES'});
+    $X = int($X);
+    $Y = int($Y);
+    $W = int($W);
+    $H = int($H); 
+    $FB->clip_rset(
+        {
+            'x'      => $X,
+            'y'      => $Y,
+            'width'  => $W,
+            'height' => $H,
+        }
+    );
     while ($RUNNING && $idx < $p) {
         my $name = $pics[$idx];
         my $image = $FB->load_image( # Uninterruptible at the moment
             {
                 'x'          => $X,
-                'y'          => $Y,
+                'y'          => ($show_name) ? $Y + 8 : $Y,
                 'width'      => $W,
-                'height'     => $H,
+                'height'     => ($show_name) ? $H - 8 : $H,
                 'file'       => $name,
                 'autolevels' => $auto
             }
@@ -422,6 +439,7 @@ sub show {
         my $tdelay = 0;
         if (ref($image) ne 'ARRAY') {
             $tdelay = $delay - $image->{'benchmark'}->{'total'};
+#            next if (exists($image->{'tags'}->{'jpeg_color_space'}) && $image->{'tags'}->{'jpeg_color_space'} <= 1);
         }
         $tdelay = 0 if ($tdelay < 0);
         if (defined($image)) {
@@ -440,16 +458,17 @@ sub show {
             }
             $FB->rbox({ 'x' => $X, 'y' => $Y, 'width' => $W, 'height' => $H, 'filled' => 1 });
             $FB->wait_for_console(); # Results will vary
+            print_it($FB, $X, $Y, basename($name)) if ($show_name);
             if (ref($image) eq 'ARRAY') {
                 my $s = time + ($delay * 2);
                 while ($RUNNING && time <= $s) {    # We play it as many times as the delay allows, but at least once.
-                                                    # We don't use "play_animation" for threads.  This is so we can stop the playback quickly.
+                    # We don't use "play_animation" for threads.  This is so we can stop the playback quickly.
                     for (my $frame = 0; $frame < scalar(@{$image}); $frame++) {
                         my $begin = time;                     # Mark the start time
                         $FB->blit_write($image->[$frame]);    # Write the frame to the display
-                                                              # Multiply the 'gif_delay' by 0.01 and then subtract from that the amount of time
-                                                              # it took to actually display the frame.  This givs the true delay, which should
-                                                              # show an accurate animation.
+                        # Multiply the 'gif_delay' by 0.01 and then subtract from that the amount of time
+                        # it took to actually display the frame.  This givs the true delay, which should
+                        # show an accurate animation.
                         my $d = (($image->[$frame]->{'tags'}->{'gif_delay'} * .01) - (time - $begin));
                         threads->yield();
                         sleep $d if ($d > 0);
@@ -469,10 +488,38 @@ sub show {
         $idx = 0 if ($idx >= $p);
     } ## end while ($RUNNING && $idx <...)
     $FB->rbox({ 'x' => $X, 'y' => $Y, 'width' => $W, 'height' => $H, 'filled' => 1 });
+    $FB->attribute_reset();
     $FB->cls('ON') if ($display);
     return(1);
 } ## end sub show
 
+sub print_it {
+    my $fb      = shift;
+    my $x       = shift;
+    my $y       = shift;
+    my $message = shift;
+    my $color   = shift || 'FFFF00FF';
+    my $bgcolor = shift || { 'red' => 0, 'green' => 0, 'blue' => 0, 'alpha' => 0 };
+
+    $fb->normal_mode();
+
+    $fb->set_b_color($bgcolor);
+    my $b = $fb->ttf_print(
+        {
+            'x'            => $x,
+            'y'            => $y+10,
+            'height'       => 8,
+            'wscale'       => 1.25,
+            'color'        => $color,
+            'text'         => $message,
+            'bounding_box' => 1,
+#            'center'       => CENTER_X,
+            'antialias'    => 0
+        }
+    );
+    $fb->ttf_print($b);
+} ## end sub print_it
+
 
 __END__
 
@@ -524,6 +571,10 @@ The program automatically determines the number of threads, and assigns two to e
 
 Keep in mind, a thread takes up memory.  So the more threads you have (and animations) the easier it is for the program to crash with an out of memory error.  This isn't a bug, just a limitation in your own system.  I have tried to make sure memory is managed as best it can be.
 
+=item B<--names>
+
+Names of each file will be printed.
+
 =back
 
 =head1 COPYRIGHT
